<?php
/**
 * WilokeHead Class
 *
 * @category Front end
 * @package Wiloke Framework
 * @author Wiloke Team
 * @version 1.0
 */

if ( !defined('ABSPATH') )
{
    exit;
}

use WilokeListGoFunctionality\Frontend\FrontendListingManagement;
use WilokeListGoFunctionality\Submit\AddListing as WilokeAddListing;

class WilokeHead
{
    public function __construct()
    {
        add_action('wp_enqueue_scripts', array($this, 'enqueue_scripts'));
        add_action('wp_head', array($this, 'wpHead'), 10);
        add_action('wp_head', array($this, 'schemaMarkup'), 10);
    }

    public function schemaMarkup(){
        global $post;
        if ( !is_singular('listing') ){
            return '';
        }
	    $planID = Wiloke::getPostMetaCaching($post->ID, wilokeRepository('app:belongsToPlanID'));
	    $aPlanSettings = Wiloke::getPostMetaCaching($planID, 'pricing_settings');

	    if ( isset($aPlanSettings['toggle_schema_markup']) && ($aPlanSettings['toggle_schema_markup']=='disable') ){
	        return '';
        }

	    $aSettings = Wiloke::getPostMetaCaching($post->ID, 'listing_settings');
        $postalAddress = '';
        if ( isset($aSettings['map']) ){
            $postalAddress = $aSettings['map']['location'];
        }

        $aLocation = wp_get_post_terms($post->ID, 'listing_location');
	    $location = '';
        if ( !is_wp_error($aLocation) && !empty($aLocation) ){
            $location = $aLocation[0]->name;
        }
        $averageRating = WilokePublic::calculateAverageRating($post);
        $averageRating = $averageRating == '0.0' ? 0 : $averageRating;
	    $aRatingInfo   = WilokeReview::calculateRating();
        ?>
        <script type="application/ld+json">
        {
            "@context" : "http://schema.org",
            "@type" : "LocalBusiness",
            "name" : "<?php echo esc_html($post->post_title); ?>",
            "description" : "<?php Wiloke::wiloke_content_limit(100, $post, false, $post->post_content, false); ?>",
            "telephone" : "<?php echo esc_attr($aSettings['phone_number']); ?>",
            "url" : "<?php echo esc_url($aSettings['website']); ?>",
            "image" : "<?php echo esc_url(get_the_post_thumbnail_url($post->ID, 'full')); ?>",
            <?php if (!empty($averageRating)) : ?>
            "aggregateRating" : {
                "@type" : "AggregateRating",
                "ratingValue" : "<?php echo esc_attr($averageRating); ?>",
                "reviewCount" : "<?php echo abs($aRatingInfo['number_of_ratings']); ?>",
                "bestRating": "5",
                "worstRating":"1"
            },
            <?php endif; ?>
            "address" : {
                "@type" : "PostalAddress",
                "streetAddress" : "<?php echo esc_attr($postalAddress); ?>",
                "addressLocality" : "<?php echo esc_attr($location); ?>"
            },
            "openingHours" : "<?php $this->generateSchemaOpeningHours($post); ?>",
            "priceRange": "<?php $this->getPriceRange($post); ?>"
        }
        </script>
        <?php
    }

    public function getPriceRange($post){
		$aSettings = Wiloke::getPostMetaCaching($post->ID, 'listing_price');
		if ( empty($aSettings) || empty($aSettings['price_segment']) ){
			return false;
		}

		if ( empty($aSettings['price_from']) && empty($aSettings['price_to']) ){
			return false;
		}

        $WilokeListGoFunctionalityApp = wiloke_listgo_functionality_get_config();
	    $currency = WilokePublic::getPaymentField('currency_code');	
	    $symbol = $WilokeListGoFunctionalityApp['currencySymbol'][$currency];
	    echo $symbol . $aSettings['price_from'] . ' - ' . $symbol . $aSettings['price_to'];
    }

    public function convertToTwentyFourFormat($hour, $format){
        if ( $format == 'PM' ){
            $hour = 12 + $hour;
        }

        return $hour;
    }

    public function generateSchemaOpeningHours($post){
	    $toggleBusinessHour = get_post_meta($post->ID, 'wiloke_toggle_business_hours', true);
	    if ( isset($toggleBusinessHour) && $toggleBusinessHour === 'disable' ){
		    return '';
	    }

        $aDays = array(
            'Mo', 'Tu', 'We', 'Th', 'Fri', 'Sa', 'Su'
        );

	    $aBusinessHours  = Wiloke::getPostMetaCaching($post->ID, 'wiloke_listgo_business_hours');

	    $beforeSH = $beforeSM = $beforeCH = $beforeCM = 0;
	    $beforeKey = 'Mo';
	    $aMarkupBNH = array();

	    if ( empty($aBusinessHours) ){
	    	return '';
	    }

	    foreach ($aBusinessHours as $key => $aBusinessHour){
	        if ( !isset($aBusinessHour['closed']) ){
		        if ( $key == 0 ){
			        $beforeSH = $this->convertToTwentyFourFormat($aBusinessHour['start_hour'], $aBusinessHour['start_format']);
			        $beforeSM = $aBusinessHour['start_minutes'];
			        $beforeCH = $this->convertToTwentyFourFormat($aBusinessHour['close_hour'], $aBusinessHour['close_format']);
			        $beforeCM = $aBusinessHour['close_minutes'];
			        $aMarkupBNH[$aDays[$key]] = $beforeSH . ':' . $beforeSM . '-' . $beforeCH . ':' . $beforeCM;
                    $beforeKey = $aDays[$key];
		        }else{
			        $currentSH = $this->convertToTwentyFourFormat($aBusinessHour['start_hour'], $aBusinessHour['start_format']);
			        $currentSM = $aBusinessHour['start_minutes'];
			        $currentCH = $this->convertToTwentyFourFormat($aBusinessHour['close_hour'], $aBusinessHour['close_format']);
			        $currentCM = $aBusinessHour['close_minutes'];
			        if ( ($currentSH == $beforeSH) && ($beforeSM == $currentSM) && ($beforeCH == $currentCH) && ( $beforeCM == $currentCM ) ){

				        $aMarkupBNH[$beforeKey.'-'.$aDays[$key]] = $aMarkupBNH[$beforeKey];
				        unset($aMarkupBNH[$beforeKey]);
				        $beforeKey = $beforeKey.'-'.$aDays[$key];
			        }else{
				        $aMarkupBNH[$aDays[$key]] = $beforeSH . ':' . $beforeSM . '-' . $beforeCH . ':' . $beforeCM;
				        $beforeSH = $currentSH;
				        $beforeSM = $currentSM;
				        $beforeCH = $currentCH;
				        $beforeCM = $currentCM;
				        $beforeKey= $aDays[$key];
			        }
		        }
            }
        }

        if ( empty($aMarkupBNH) ){
	        return '';
        }

        $aProcessMarkup = array();
        foreach ( $aMarkupBNH as $range => $bsH ){
	        $aProcessMarkup[] = $range . ' ' . $bsH;
        }

        echo '[' . implode(',', $aProcessMarkup) . ']';
    }

    public function wpHead(){
	    global $wiloke, $post;
	    if ( empty($wiloke) ) {
		    return;
	    }

	    if ( !function_exists('wp_site_icon') || ( function_exists('has_site_icon') && !has_site_icon() ) ) {
		    if ( isset($wiloke->aThemeOptions['general_favicon']['thumbnail']) && !empty($wiloke->aThemeOptions['general_favicon']['thumbnail']) ){
			    ?><link rel="shortcut icon" href="<?php echo esc_url($wiloke->aThemeOptions['general_favicon']['thumbnail']); ?>"><?php
		    }
	    }
	    if( isset($wiloke->aThemeOptions['seo_open_graph_meta']) && $wiloke->aThemeOptions['seo_open_graph_meta'] === 'enable' && !is_404() && !empty($post) ) :
		    if( is_singular() ){
			    $aPageSettings = Wiloke::getPostMetaCaching($post->ID, 'single_page_settings');
			    if( isset($aPageSettings['page_description']) ){
				    $desc = esc_attr( $aPageSettings['page_description'] );
			    }else if( !empty( $post->post_excerpt ) ){
				    $desc = esc_attr( wp_trim_words( $post->post_excerpt, 50 ) );
			    }else if( strpos($post->post_content, '[vc_row') === false ){
				    $desc = esc_attr( wp_trim_words( $post->post_content, 50 ) );
			    }else{
				    $desc = esc_attr( $post->post_title );
			    }
			    $metaImage = get_the_post_thumbnail_url($post->ID, 'large');
			    $title = $post->post_title;
			    $link = get_permalink( $post->ID );
		    }else if ( is_author() ){
			    $aAuthor = Wiloke::getUserMeta(get_queried_object_id());
			    $desc = $aAuthor['description'];
			    $link = get_author_posts_url(get_queried_object_id());

		    }else if ( is_tax() ){
			    $desc = term_description(get_queried_object_id(), get_query_var( 'taxonomy' ));
		     	$aOptions = Wiloke::getTermOption(get_queried_object_id());
			    $metaImage = isset($aOptions['featured_image']) ? wp_get_attachment_image_url($aOptions['featured_image'], 'large') : '';
		    	$title = single_term_title('', false);
		    	$link = get_term_link(get_queried_object_id(), get_query_var( 'taxonomy' ));
		    }

		    $title = isset($title) ? $title : get_bloginfo('name');
		    $desc = !isset($desc) || empty($desc) ? get_option('blogdescription') : $desc;
		    $link = isset($link) ? $link : home_url('/');
		    ?>
		    <?php
		    $aFbSettings = Wiloke::getOption('_wiloke_facebook_settings');
		    if ( isset($aFbSettings['app_id']) && !empty($aFbSettings['app_id']) ) :
			    ?>
                <meta property="fb:app_id" content="<?php echo esc_attr($aFbSettings['app_id']) ?>">
		    <?php endif; ?>

            <meta property="og:type" content="website" />
            <meta property="og:url" content="<?php echo esc_url($link); ?>" />
            <meta property="og:title" content="<?php echo esc_attr( $title ); ?>" />
            <meta property="og:description" content="<?php echo esc_attr(strip_shortcodes($desc)); ?>">
		    <?php
		    if ( !isset($metaImage)|| !$metaImage ) {
		    	$metaImage = !empty($wiloke->aThemeOptions['seo_og_image']['id']) ? wp_get_attachment_image_url($wiloke->aThemeOptions['seo_og_image']['id'], 'large') : '';
		    }
		    if ( !empty($metaImage) ) {
			    echo '<meta property="og:image" content="'.esc_url($metaImage).'" />';
		    }
	    endif;
    }
    
    public function enqueue_scripts()
    {
        wp_enqueue_style('wiloke-alert', Wiloke::$public_url . 'source/css/alert.css', array(), WILOKE_THEMEVERSION);

        global $post, $current_user, $wiloke;

        if ( isset($wiloke->aThemeOptions['toggle_google_recaptcha']) && $wiloke->aThemeOptions['toggle_google_recaptcha'] == 'enable' && !is_user_logged_in() ){
            wp_enqueue_script('google-recaptcha', 'https://www.google.com/recaptcha/api.js', array(), null, false);
        }

        $aUserInfo = array();
        if ( !empty($current_user) ){
            $aUserInfo = array(
                'user_id'               => $current_user->ID,
                'author_name'           => $current_user->user_nicename,
                'user_url'              => $current_user->user_url,
                'author_avatar_urls'    => array(
                    96 => get_avatar_url($current_user->ID, array('size'=>96))
                )
            );
        }
        $mapPageUrl = isset($wiloke->aThemeOptions['header_search_map_page']) ? get_permalink($wiloke->aThemeOptions['header_search_map_page']) : '';
        $mapToken = isset($wiloke->aThemeOptions['general_mapbox_api']) ? $wiloke->aThemeOptions['general_mapbox_api'] : '';
        $mapTheme = isset($wiloke->aThemeOptions['general_map_theme']) ? $wiloke->aThemeOptions['general_map_theme'] : '';
        $mapMaxZoom = isset($wiloke->aThemeOptions['general_map_max_zoom']) && !empty($wiloke->aThemeOptions['general_map_max_zoom']) ? $wiloke->aThemeOptions['general_map_max_zoom'] : 4;
        $mapMinZoom = isset($wiloke->aThemeOptions['general_map_min_zoom']) && !empty($wiloke->aThemeOptions['general_map_min_zoom']) ? $wiloke->aThemeOptions['general_map_min_zoom'] : -1;
        $mapClusterRadius = isset($wiloke->aThemeOptions['general_map_cluster_radius']) && !empty($wiloke->aThemeOptions['general_map_cluster_radius']) ? $wiloke->aThemeOptions['general_map_cluster_radius'] : 60;
	    $mapCenterZoom = isset($wiloke->aThemeOptions['general_map_center_zoom']) && !empty($wiloke->aThemeOptions['general_map_center_zoom']) ? $wiloke->aThemeOptions['general_map_center_zoom'] : 10;
        $toggleAskForPosition = isset($wiloke->aThemeOptions['listing_toggle_ask_for_geocode']) && !empty($wiloke->aThemeOptions['listing_toggle_ask_for_geocode']) ? $wiloke->aThemeOptions['listing_toggle_ask_for_geocode'] : 'enable';
        $googleAPIKey = isset($wiloke->aThemeOptions['general_map_api']) && !empty($wiloke->aThemeOptions['general_map_api']) ? $wiloke->aThemeOptions['general_map_api'] : '';
        $paymentMode = WilokePublic::getPaymentField('mode');

	    $mapSingleMaxZoom = isset($wiloke->aThemeOptions['general_map_single_max_zoom']) && !empty($wiloke->aThemeOptions['general_map_single_max_zoom']) ? $wiloke->aThemeOptions['general_map_single_max_zoom'] : 4;
	    $mapSingleMinZoom = isset($wiloke->aThemeOptions['general_map_min_zoom']) && !empty($wiloke->aThemeOptions['general_map_single_min_zoom']) ? $wiloke->aThemeOptions['general_map_single_min_zoom'] : -1;
	    $mapSingleCenterZoom = isset($wiloke->aThemeOptions['general_map_single_center_zoom']) && !empty($wiloke->aThemeOptions['general_map_single_center_zoom']) ? $wiloke->aThemeOptions['general_map_single_center_zoom'] : 60;
	    $toggleBusinessListingHour = isset($wiloke->aThemeOptions['listing_toggle_business_hours']) && !empty($wiloke->aThemeOptions['listing_toggle_business_hours']) ? $wiloke->aThemeOptions['listing_toggle_business_hours'] : 'enable';

	    $googleMapThemeType = isset($wiloke->aThemeOptions['general_map_gg_map_theme']) ? $wiloke->aThemeOptions['general_map_gg_map_theme'] : 'blurWater';
	    if ( $googleMapThemeType == 'custom' ){
	    	$googleMapThemeCode = $wiloke->aThemeOptions['general_map_gg_map_theme_code'];
	    }else{
		    $googleMapThemeCode = '[]';
        }

        if ( $paymentMode != 'sandbox' ){
            $twoCheckoutMode = 'production';
	        $twoCheckOutSellerID = WilokePublic::getPaymentField('2co_live_seller_id');
	        $twoCheckOutPublishableKey = WilokePublic::getPaymentField('2co_live_publishable_key');
        }else{
	        $twoCheckOutSellerID = WilokePublic::getPaymentField('2co_sandbox_seller_id');
	        $twoCheckOutPublishableKey = WilokePublic::getPaymentField('2co_sandbox_publishable_key');
	        $twoCheckoutMode = 'sandbox';
        }

	    $toggleEmbedVideo = '';
	    $toggleListingShortcodes = '';
        if ( isset($_REQUEST['package_id']) ){
	        $aPackageSettings = WilokeAddListing::packageAllow();
	        $toggleListingShortcodes = isset($aPackageSettings['toggle_listing_shortcode']) ? $aPackageSettings['toggle_listing_shortcode'] : 'enable';
	        $toggleEmbedVideo = isset($aPackageSettings['toggle_allow_embed_video']) ? $aPackageSettings['toggle_allow_embed_video'] : 'enable';
        }

        $disableFetchNotification = 'no';
        if ( (isset($_REQUEST['mode']) && $_REQUEST['mode'] == 'preview') || ( class_exists('WilokeListgoFunctionality\Framework\Payment\PaymentConfiguration') &&  ( isset($post->ID) && ( ($post->ID == \WilokeListgoFunctionality\Framework\Payment\PaymentConfiguration::getField('addlisting')) || ($post->ID == \WilokeListgoFunctionality\Framework\Payment\PaymentConfiguration::getField('checkout')) || ($post->ID == \WilokeListgoFunctionality\Framework\Payment\PaymentConfiguration::getField('package')))  )
            ) ){
	        $disableFetchNotification = 'yes';
        }

        $countryRestriction = isset($wiloke->aThemeOptions['listing_country_restriction']) ? $wiloke->aThemeOptions['listing_country_restriction'] : '';

        wp_localize_script('jquery-migrate', 'WILOKE_LISTGO_TRANSLATION', apply_filters(
            'wiloke/listgo/general_transaltion', array(
		        'deniedaccess'      => esc_html__('You don\'t have permission to access this page.', 'listgo'),
		        'selectoptions'     => esc_html__('Select an option', 'listgo'),
		        'requirelocation'   => esc_html__('Please enter in your location', 'listgo'),
		        'report'            => esc_html__('Could you please tell us why this article should not display on our website?', 'listgo'),
		        'opennow'           => esc_html__('Open now', 'listgo'),
		        'closednow'         => esc_html__('Closed now', 'listgo'),
		        'location'          => esc_html__('Location', 'listgo'),
		        'address'           => esc_html__('Address', 'listgo'),
		        'phone_number'      => esc_html__('Phone', 'listgo'),
		        'contact_email'     => esc_html__('Email', 'listgo'),
		        'website'           => esc_html__('Website', 'listgo'),
		        'email'             => esc_html__('Contact Us', 'listgo'),
		        'addtofavorite'     => esc_html__('Favorite', 'listgo'),
		        'save'              => esc_html__('Save', 'listgo'),
		        'following'         => esc_html__('Follow this author', 'listgo'),
		        'ajaxerror'         => esc_html__('Something went wrong', 'listgo'),
		        'needsingup'        => esc_html__('Please sign up to follow this author', 'listgo'),
		        'notallowfollow'    => esc_html__('OOps! You can not follow yourself', 'listgo'),
		        'usernotexists'     => esc_html__('This user does not exist.', 'listgo'),
		        'followsuccess'     => esc_html__('Thanks for your following. You will receive each new post by email.', 'listgo'),
		        'signuptofollow'    => esc_html__('Sign up to follow this author\'s article', 'listgo'),
		        'isfollowing'       => esc_html__('You are following the author\'s article', 'listgo'),
		        'signupbutnotemail' => esc_html__('Please supply your email address', 'listgo'),
		        'finddirections'    => esc_html__('Find directions', 'listgo'),
		        'gotomap'           => esc_html__('Go To Map', 'listgo'),
		        'viewdetail'        => esc_html__('View Detail', 'listgo'),
		        'next'              => esc_html__('Next', 'listgo'),
		        'prev'              => esc_html__('Prev', 'listgo'),
		        'notfound'          => esc_html__('Sorry, we couldn\'t find what you\'re looking for. Please try another search.', 'listgo'),
		        'somethingwrong'    => esc_html__('Oops! Something went wrong. Please contact the administrator to report this issue', 'listgo'),
		        'packagenotexist'   => esc_html__('The package does not exists.', 'listgo'),
		        'emailexisted'      => esc_html__('The email address you entered is already in use on another account', 'listgo'),
		        'passwdrequired'    => esc_html__('Please enter your password.', 'listgo'),
		        'signupfail'        => esc_html__('Wrong username or password.', 'listgo'),
		        'wrongemail'        => esc_html__('You entered a wrong email, please try another one.', 'listgo'),
		        'titlerequired'     => esc_html__('The title is required', 'listgo'),
		        'contentrequired'   => esc_html__('The content is required', 'listgo'),
		        'packagerequired'   => esc_html__('Please select a package before submitting', 'listgo'),
		        'deninedsubmission' => esc_html__('Your submission has been rejected. Reason: The post id does not exist or you are not author of this article', 'listgo'),
		        'isreviewing'       => esc_html__('We are reviewing this article so You can not edit it at the time. Please be patient and waiting an email from us', 'listgo'),
		        'checkoutwrong'     => esc_html__('Checkout page is not found. Please contact the site manager about this issue.', 'listgo'),
		        'securitycodewrong' => esc_html__('Security code is wrong', 'listgo'),
		        'readmore'          => esc_html__('Read more', 'listgo'),
		        'getdirections'     => esc_html__('Get directions', 'listgo'),
		        'followingtext'     => esc_html__('Following', 'listgo'),
		        'unfollowingtext'   => esc_html__('Follow', 'listgo'),
		        'pinned'            => esc_html__('Pinned Listing', 'listgo'),
		        'unpinned'          => esc_html__('Pin To Top', 'listgo'),
		        'geocodefailed'     => esc_html__('Geocoder failed due to:', 'listgo'),
		        'noresults'         => esc_html__('No results found', 'listgo'),
		        'publish'           => esc_html__('Published', 'listgo'),
		        'pending'           => esc_html__('Pending', 'listgo'),
		        'expired'           => esc_html__('Expired', 'listgo'),
		        'more'				=> esc_html__('+ More', 'listgo'),
		        'confirmCancelPlan'	=> esc_html__('Are you sure want to cancel this plan', 'listgo')
	        )
        ));

        ob_start();
        ?>
        var wilokeVisited = 0;
        if ( typeof WILOKE_GLOBAL === 'undefined' )
        {
            window.WILOKE_GLOBAL                = {};
            WILOKE_GLOBAL.isLoggedIn            = '<?php echo is_user_logged_in() ? "yes" : "no"; ?>';
            WILOKE_GLOBAL.isDisableFetchNotification = '<?php echo esc_js($disableFetchNotification); ?>';
            WILOKE_GLOBAL.userInfo              = '<?php echo json_encode($aUserInfo); ?>';
            WILOKE_GLOBAL.homeURI               = '<?php echo esc_js(urlencode(get_template_directory_uri() . '/')); ?>';
            WILOKE_GLOBAL.blogname              = '<?php echo esc_attr(get_option('blogname')); ?>';
            WILOKE_GLOBAL.toggleBusinessListingHour  = '<?php echo esc_attr($toggleBusinessListingHour); ?>';
            WILOKE_GLOBAL.wiloke_nonce          = '<?php echo esc_js(wp_create_nonce('wiloke-nonce')); ?>';
            WILOKE_GLOBAL.ajaxurl               = '<?php echo esc_url(admin_url("admin-ajax.php")); ?>';
            WILOKE_GLOBAL.postID                = <?php echo isset($post->ID) ? esc_js($post->ID) : -1; ?>;
            WILOKE_GLOBAL.authorID              = <?php echo isset($post->post_author) ? esc_js($post->post_author) : -1; ?>;
            WILOKE_GLOBAL.post_type             = '<?php echo isset($post->post_type) ? esc_js($post->post_type) : -1; ?>';
            WILOKE_GLOBAL.portfolio_data        = {};
            WILOKE_GLOBAL.portfolio_loaded_cats = {};
            WILOKE_GLOBAL.mappage = '<?php echo esc_url($mapPageUrl); ?>';
            WILOKE_GLOBAL.twoCheckoutSellerID = '<?php echo esc_attr($twoCheckOutSellerID); ?>';
            WILOKE_GLOBAL.twoCheckoutPublishableKey = '<?php echo esc_attr($twoCheckOutPublishableKey); ?>';
            WILOKE_GLOBAL.googleapikey  = '<?php echo esc_js($googleAPIKey); ?>';
            WILOKE_GLOBAL.googleMapThemeType  = '<?php echo esc_js($googleMapThemeType); ?>';
            WILOKE_GLOBAL.googleMapThemeCode  = <?php echo $googleMapThemeCode; ?>;
            WILOKE_GLOBAL.maptoken  = '<?php echo esc_js($mapToken); ?>';
            WILOKE_GLOBAL.maptheme  = '<?php echo esc_js($mapTheme); ?>';
            WILOKE_GLOBAL.mapmaxzoom  = <?php echo esc_js($mapMaxZoom); ?>;
            WILOKE_GLOBAL.mapminzoom  = <?php echo esc_js($mapMinZoom); ?>;
            WILOKE_GLOBAL.centerZoom  = <?php echo esc_js($mapCenterZoom); ?>;
            WILOKE_GLOBAL.mapSingleMaxZoom  = <?php echo esc_js($mapSingleMaxZoom); ?>;
            WILOKE_GLOBAL.mapSingleMinZoom  = <?php echo esc_js($mapSingleMinZoom); ?>;
            WILOKE_GLOBAL.mapSingleCenterZoom  = <?php echo esc_js($mapSingleCenterZoom); ?>;
            WILOKE_GLOBAL.mapcluster  = <?php echo esc_js($mapClusterRadius); ?>;
            WILOKE_GLOBAL.toggleAskForPosition  = '<?php echo esc_js($toggleAskForPosition); ?>';
            WILOKE_GLOBAL.posts__not_in         = {};
            WILOKE_GLOBAL.woocommerce           = {};
            WILOKE_GLOBAL.is_debug = '<?php echo defined('WP_DEBUG') && WP_DEBUG ? 'true' : 'false'; ?>';
            WILOKE_GLOBAL.siteCache = {};
            WILOKE_GLOBAL.countryRestriction = '<?php echo esc_js($countryRestriction); ?>';
            WILOKE_GLOBAL.toggleListingShortcodes = '<?php echo esc_attr($toggleListingShortcodes); ?>';
            WILOKE_GLOBAL.toggleEmbedVideo = '<?php echo esc_attr($toggleEmbedVideo); ?>';
            WILOKE_GLOBAL.twocheckoutMode = '<?php echo esc_attr($twoCheckoutMode); ?>';
            WILOKE_GLOBAL.isLazyLoad = '<?php echo !isset($wiloke->aThemeOptions['general_is_lazy_load']) || $wiloke->aThemeOptions['general_is_lazy_load'] == 'yes' ? 'yes' : 'no'; ?>';
        	WILOKE_GLOBAL.datepickerFormat = '<?php echo esc_js(apply_filters('listgo/date-picker-format', 'mm/dd/yy')) ?>'
        }
        <?php
        $content = ob_get_contents();
        ob_end_clean();
        wp_add_inline_script('jquery-migrate', $content);
    }
}