<?php
use WilokeListgoFunctionality\AlterTable\AlterTableGeoPosition;
use WilokeListGoFunctionality\AlterTable\AlterTableBusinessHours as WilokeBusinessHoursTbl;
use WilokeListgoFunctionality\Frontend\FrontendRating;
use WilokeListGoFunctionality\AlterTable\AlterTablePriceSegment as WilokePriceSegmentTbl;
use WilokeListgoFunctionality\AlterTable\AlterTableEventsCalendar;
use WilokeListgoFunctionality\Framework\Helpers\Time;

class WilokeSearchSystem{
	public function __construct() {
		add_action('wp_ajax_wiloke_search_suggestion', array($this, 'searchSuggestion'));
		add_action('wp_ajax_nopriv_wiloke_search_suggestion', array($this, 'searchSuggestion'));

		add_action('wp_ajax_fetch_quick_search_form', array($this, 'fetchQuickSearchForm'));
		add_action('wp_ajax_nopriv_fetch_quick_search_form', array($this, 'fetchQuickSearchForm'));

		add_filter('query_vars', array($this, 'customQuery'));

		add_filter('posts_join', array($this, 'joinLatLng'), 10, 2);
		add_filter('posts_pre_query', array($this, 'addHavingDistance'), 10, 99);
		add_filter('posts_fields', array($this, 'addLatLngSelectionToQuery'), 10, 2);
		add_filter('posts_orderby', array($this, 'orderByDistance'), 10, 2);

		add_filter('posts_join', array($this, 'joinOpenNow'), 15, 2);
		add_filter('posts_where' , array($this, 'addOpenNowToPostsWhere'), 15, 2);

		add_filter('posts_join', array($this, 'joinPriceRange'), 15, 2);
		add_filter('posts_where' , array($this, 'addPriceRangeToQuery'), 15, 2);

		add_filter('posts_join', array($this, 'joinEvents'), 10, 2);
		add_filter('posts_orderby', array($this, 'orderByEventDate'), 10, 2);
		add_filter('posts_where', array($this, 'addEventWhen'), 10, 2);
		add_filter('posts_fields', array($this, 'addEventSelection'), 10, 2);
	}

	private function isAdminQuery(){
		if ( is_admin() && !wp_doing_ajax() ){
			return true;
		}

		if ( wp_doing_ajax() ){
			if ( isset($_REQUEST['action']) && (strpos($_REQUEST['action'], 'listgo') != false || strpos($_REQUEST['action'], 'wiloke') !== false)  ){
				return false;
			}
		}

		return true;
	}

	private function orderByEventConditional($that){
		return !isset($that->query_vars['orderby']) || empty($that->query_vars['orderby']) || (!is_array($that->query_vars['orderby']) && strpos($that->query_vars['orderby'], 'event_') === false) ? false : true;
	}

	private function isQueryEvent($that){
		return $that->query_vars['post_type'] == 'event';
	}

	public function addEventSelection($field, $that){
	    if ( !class_exists('WilokeListgoFunctionality\AlterTable\AlterTableEventsCalendar') || $this->isAdminQuery() ){
	        return $field;
        }

		if ( $this->orderByEventConditional($that) || $this->isQueryEvent($that) ){
			global $wpdb;
			$eventDataTbl = $wpdb->prefix . AlterTableEventsCalendar::$tblName;
			$field .= ", $eventDataTbl.startsGMT as event_starts_on, $eventDataTbl.endsOnGMT";

		}
		return $field;
	}

	public function orderByEventDate($orderBy, $that){
		if ( ( !$this->orderByEventConditional($that) && !$this->isQueryEvent($that) ) || $this->isAdminQuery() ){
			return $orderBy;
		}

		return  "event_starts_on ASC";
	}

	public function addEventWhen($where, $that){
		if ( !class_exists('WilokeListgoFunctionality\AlterTable\AlterTableEventsCalendar') || $this->isAdminQuery() ){
			return $where;
		}
		
		global $wpdb;
		$eventTbl = $wpdb->prefix . AlterTableEventsCalendar::$tblName;
		date_default_timezone_set('UTC');
		$starts = date('Y-m-d', time()) . ' 23:59:00';
		$now = current_time('mysql', 1);


	    if ( $that->query_vars['post_type'] == 'event' ){
		    $where .= " AND ( $eventTbl.endsOnGMT >= '".$now."') ";
        }

		if ( !$this->orderByEventConditional($that) ){
			return $where;
		}


		$where .= " AND ( ";
		if ( $that->query_vars['orderby'] == 'event_today' || $that->query_vars['orderby'] == 'event_this_weekend' || $that->query_vars['orderby'] == 'event_tomorrow' ){
			if ( $that->query_vars['orderby'] == 'event_tomorrow' ){
				$starts = date('Y-m-d H:i:s', strtotime($starts . ' + 1 days'));
			}elseif ( $that->query_vars['orderby'] == 'event_this_weekend' ){
				$starts = date('Y-m-d H:i:s', strtotime('sunday this week'));
				$now = $starts;
			}else{
				$starts = $now;
			}

			$where .= $wpdb->prepare("%s >= $eventTbl.startsGMT AND $eventTbl.endsOnGMT >= %s", $starts, $now);
		}else if( $that->query_vars['orderby'] == 'event_upcoming_event' ){
			$where .= " $eventTbl.startsGMT > '".$now."'";
		}else if ($that->query_vars['orderby'] == 'event_ongoing_event'){
			$where .= " $eventTbl.startsGMT <= '".$now."' AND $eventTbl.endsOnGMT >= '".$now."'";
		}else if ( $that->query_vars['orderby'] == 'event_expired_event' ){
			$where .= " ($eventTbl.endsOnGMT <= '".$now."' || $eventTbl.endsOnGMT <= $eventTbl.startsGMT)";
		}else{
			if ( $that->query_vars['orderby'] == 'event_next_week' ){
				$starts = date('Y-m-d H:i:s', strtotime('monday next week'));
				$ends = date('Y-m-d H:i:s', strtotime('sunday next week'));
			}else if($that->query_vars['orderby'] == 'event_this_week'){
				$starts = date('Y-m-d H:i:s', strtotime('monday this week'));
				$ends = date('Y-m-d H:i:s', strtotime('sunday this week'));
			}else if($that->query_vars['orderby'] == 'event_next_month'){
				$starts = date('Y-m-d H:i:s', strtotime('first day of next month'));
				$ends = date('Y-m-d H:i:s', strtotime('last day of next month'));
			}else{
				$starts = date('Y-m-d H:i:s', strtotime('first day of next week'));
				$ends = date('Y-m-d H:i:s', strtotime('last day of next week'));
			}

			$where .= $wpdb->prepare(" $eventTbl.endsOnGMT <= %s AND $eventTbl.endsOnGMT >= %s AND $eventTbl.endsOnGMT >= %s", $ends, $now, $starts);
        }

		$where .= ")";
		return $where;
	}

	public function joinEvents($join, $that){
		if ( !class_exists('WilokeListgoFunctionality\AlterTable\AlterTableEventsCalendar') || $this->isAdminQuery() ){
			return $join;
		}

		global $wpdb;
		if ( !$this->orderByEventConditional($that) && !$this->isQueryEvent($that) ){
			return $join;
		}

		$eventsDataTbl = $wpdb->prefix . AlterTableEventsCalendar::$tblName;

		$join .= " LEFT JOIN $eventsDataTbl ON ($eventsDataTbl.eventID = $wpdb->posts.ID)";
		return $join;
    }

	public function joinPriceRange($join, $that){
		global $wpdb;
		if ( $this->isAdminQuery() ){
			return $join;
		}

		if ( isset($that->query_vars['price_segment']) && !empty($that->query_vars['price_segment']) ){
			$priceSegmentTbl = $wpdb->prefix . WilokePriceSegmentTbl::$tblName;
			if ( strpos($join, $priceSegmentTbl) === false ){
				$priceRange = " LEFT JOIN $priceSegmentTbl ON ($priceSegmentTbl.post_ID = $wpdb->posts.ID)";
				$join .= $priceRange;
			}
        }
        return $join;
	}

	public function addPriceRangeToQuery($where, $that){
		global $wpdb;
		if ( $this->isAdminQuery() ){
			return $where;
		}
		if ( isset($that->query_vars['price_segment']) && !empty($that->query_vars['price_segment']) ){
			$priceSegmentTbl = $wpdb->prefix . WilokePriceSegmentTbl::$tblName;
			$where .= " AND " . $wpdb->prepare(
				"$priceSegmentTbl.segment=%s",
				$that->query_vars['price_segment']
			);
        }
        return $where;
	}

	public function getTimezoneByGeoCode($geocode){
		global $wiloke;
		$aTimeZone = wp_remote_get(esc_url_raw('https://maps.googleapis.com/maps/api/timezone/json?location='.$geocode.'&timestamp='.time().'&key='.$wiloke->aThemeOptions['general_map_api']));

		if ( is_wp_error($aTimeZone)  ){
			return '';
		}else{
			$oTimeZone = json_decode($aTimeZone['body']);
			return $oTimeZone->timeZoneId;
		}
	}

	private function getListingLocations($that){
		if ( !empty($that->query_vars['tax_query']) ){
			foreach ($that->query_vars['tax_query'] as $aTaxQuery) {
				if ( $aTaxQuery['taxonomy'] == 'listing_location' ){
					return $aTaxQuery['term_id'];
				}
			}
		}

		return false;
	}

	public function addOpenNowToPostsWhere($where, $that){
		if ( !class_exists('WilokeListgoFunctionality\AlterTable\AlterTableBusinessHours') || $this->isAdminQuery() ){
			return $where;
		}

		global $wpdb;
		$businessHoursTbl = $wpdb->prefix . WilokeBusinessHoursTbl::$tblName;
		if ( isset($that->query_vars['open_now']) && !empty($that->query_vars['open_now']) ){
			$timezone = '';
		    if ( isset($that->query_vars['geocode']) && !empty($that->query_vars['geocode']) ){
				$timezone = $this->getTimezoneByGeoCode($that->query_vars['geocode']['latLng']);
			}

			if ( empty($timezone) ){
				$aLocationIDs = $this->getListingLocations($that);
				if ( is_array($aLocationIDs) ){
					foreach ($aLocationIDs as $locationID) {
						$aLocationSettings = Wiloke::getTermOption($locationID);
						if ( !empty($aLocationSettings['timezone']) ){
							$timezone = $aLocationSettings['timezone'];
							break;
						}
					}
                }
			}

			if ( !isset($timezone) || strpos($timezone, 'UTC') !== false ){
				$parseUTC = str_replace('UTC', '', $timezone);
				date_default_timezone_set('UTC');
				$now = date('H:i:s', strtotime($parseUTC . ' hours'));
				$today = date('N', strtotime($parseUTC . ' hours'));
				$today = $today - 1;
			}else{
				date_default_timezone_set($timezone);
				$now = date('H:i:s', time());
				$today = date('N', time());
				$today = $today - 1;
			}

			$where .= " AND " . $wpdb->prepare("
					(($businessHoursTbl.always_open = %s) 
					OR  
					( 
						($businessHoursTbl.extra_open_time = '00:00:00' OR $businessHoursTbl.extra_close_time = '00:00:00' || $businessHoursTbl.extra_open_time IS NULL || $businessHoursTbl.extra_close_time IS NULL) 
						AND
						$businessHoursTbl.day_of_week = %d AND (  
							( %s >= $businessHoursTbl.open_time AND $businessHoursTbl.close_time > %s) 
							OR  ( 
								$businessHoursTbl.close_time < $businessHoursTbl.open_time 
								AND ( 
									(%s > $businessHoursTbl.open_time) 
									OR (%s >= %s AND $businessHoursTbl.close_time < %s AND $businessHoursTbl.close_time > %s) 
								)
							)
						)
						AND $businessHoursTbl.closed != 1
					)
					OR 
					(
						($businessHoursTbl.extra_open_time IS NOT NULL AND $businessHoursTbl.extra_close_time IS NOT NULL AND $businessHoursTbl.extra_open_time != '00:00:00' AND $businessHoursTbl.extra_close_time !='00:00:00') 
						AND
						(
							$businessHoursTbl.day_of_week = %d 
							AND (  
								( %s >= $businessHoursTbl.open_time AND $businessHoursTbl.close_time > %s) 
								OR 
								(
									%s >= $businessHoursTbl.extra_open_time AND $businessHoursTbl.extra_close_time > %s
								)
								OR 
								(  
									$businessHoursTbl.extra_close_time < $businessHoursTbl.extra_open_time 
									AND (
										(%s > $businessHoursTbl.extra_open_time) 
										OR (%s >= %s AND $businessHoursTbl.extra_close_time < %s AND $businessHoursTbl.extra_close_time > %s) 
									)
								)
							)
						)
						AND $businessHoursTbl.closed != 1
					))",
				'yes', $today, $now, $now, $now, $now, '00:00:00', '06:00:00', $now, $today, $now, $now, $now, $now, $now, $now, '00:00:00', '06:00:00', $now );
		}

		return $where;
	}

	public function joinOpenNow($join, $that){
		global $wpdb;
		if ( !class_exists('WilokeListgoFunctionality\AlterTable\AlterTableBusinessHours') || $this->isAdminQuery() ){
			return $join;
		}

		if ( isset($that->query_vars['open_now']) && !empty($that->query_vars['open_now']) ){
			$businessHoursTbl = $wpdb->prefix . WilokeBusinessHoursTbl::$tblName;
			$joinOpenNow = " LEFT JOIN $businessHoursTbl ON ($businessHoursTbl.post_ID = $wpdb->posts.ID)";

			if ( strpos($join, $joinOpenNow) === false ){
				$join .= $joinOpenNow;
			}
        }
        return $join;
	}

	public function orderByDistance($orderBy, $that){
		if ( $this->isAdminQuery() ){
			return $orderBy;
		}

		if ( isset($that->query_vars['geocode']) && !empty($that->query_vars['geocode']) && ( !isset($that->query_vars['is_highest_rated']) ) ){
			return  'wiloke_distance';
		}
       	return $orderBy;
	}
	public function addLatLngSelectionToQuery($field, $that){
		if ( $this->isAdminQuery() ){
			return $field;
		}

	    if ( isset($that->query_vars['geocode']) && !empty($that->query_vars['geocode']) ){
	        global $wpdb;
		    $latLngTbl = $wpdb->prefix . AlterTableGeoPosition::$tblName;

		    $unit = esc_sql($that->query_vars['geocode']['unit']);
		    $aParseLatLng = explode(',', $that->query_vars['geocode']['latLng']);
		    $unit = $unit == 'km' ? 6371 : 3959;
		    $lat = esc_sql(trim($aParseLatLng[0]));
		    $lng = esc_sql(trim($aParseLatLng[1]));

		    $field .= $wpdb->prepare(
			    ",( %d * acos( cos( radians(%s) ) * cos( radians( $latLngTbl.lat ) ) * cos( radians( $latLngTbl.lng ) - radians(%s) ) + sin( radians(%s) ) * sin( radians( $latLngTbl.lat ) ) ) ) as wiloke_distance",
                $unit, $lat, $lng, $lat
            );
	    }
        return $field;
    }

	public function customQuery($aVars){
		$aVars[] = 'geocode';
		$aVars[] = 'open_now';
		$aVars[] = 'highest_rated';
		return $aVars;
    }

	public function joinLatLng($join, $that){
		global $wpdb;
		if ( $this->isAdminQuery() ){
			return $join;
		}

		if ( ( (isset($that->query_vars['geocode']) && !empty($that->query_vars['geocode']) ) || ( isset($that->query_vars['map_bounds']) && !empty($that->query_vars['map_bounds']) ) ) ){
			$latLngTbl = $wpdb->prefix . AlterTableGeoPosition::$tblName;
			$joinLatLng = " LEFT JOIN $latLngTbl ON ($latLngTbl.postID = $wpdb->posts.ID)";
			if ( strpos($join, $joinLatLng) === false ){
				$join .= $joinLatLng;
			}
        }
        return $join;
    }

	public function addHavingDistance($nothing, $that){
		if ( $this->isAdminQuery() ){
			return $nothing;
		}

		if ( isset($that->query_vars['geocode']) && !empty($that->query_vars['geocode']) ){
			$that->request = str_replace('ORDER BY', 'HAVING wiloke_distance < '. abs($that->query_vars['geocode']['radius']) . ' ORDER BY', $that->request);
		}

		return $nothing;
	}

	public function fetchQuickSearchForm(){
        $aOptions = get_option('wiloke_themeoptions');
        if ( !isset($aOptions['toggle_quick_search_form']) || $aOptions['toggle_quick_search_form'] == 'disable' ){
            wp_send_json_error(array(
                'msg' => esc_html__('Please go to Appearance -> Theme Options -> Search Form Settings to enable this feature', 'wiloke-listgo-functionality')
            ));
        }

        $keyword = '%'.esc_sql($_POST['keyword']).'%';

        global $wpdb;
        $aResults = $wpdb->get_results(
            $wpdb->prepare(
                "SELECT $wpdb->posts.ID FROM $wpdb->posts WHERE post_title LIKE %s AND post_status='publish' AND post_type='listing' ORDER BY IF(menu_order!=0, menu_order, post_date) LIMIT 15 ",
	            $keyword
            )
        );

		$total = 0;
		$aPosts = array();

        if ( !empty($aResults) && !is_wp_error($aResults) ){
            $total = count($aResults);
        }

		$defaultFeaturedImg = '';
		if ( isset($aOptions['listing_header_image']) && isset($aOptions['listing_header_image']['id']) && !empty($aOptions['listing_header_image']['id']) ){
			$defaultFeaturedImg = wp_get_attachment_image_url($aOptions['listing_header_image']['id'], 'thumbnail');
		}

		foreach ($aResults as $oResult){
			$featuredImg = get_the_post_thumbnail_url($oResult->ID, 'thumbnail');
			$aPosts[] = array(
				'link' => get_permalink($oResult->ID),
				'title'=> get_the_title($oResult->ID),
				'img'  => empty($featuredImg) ? $defaultFeaturedImg : $featuredImg
			);
		}

        if ( $total == 15  ){
            wp_send_json_success(array(
               'msg' => $aPosts
            ));
        }

        $rest = 15 - $total;

        $termsTbl = $wpdb->terms;
        $termTaxTbl = $wpdb->term_taxonomy;

		$aResults = $wpdb->get_results(
			$wpdb->prepare(
				"SELECT $termsTbl.term_id FROM $termsTbl LEFT JOIN $termTaxTbl ON ($termTaxTbl.term_id = $termsTbl.term_id) WHERE $termsTbl.name LIKE %s AND $termTaxTbl.taxonomy='listing_tag' LIMIT 10",
				$keyword
            )
		);

		if ( !empty($aResults) && !is_wp_error($aResults) ){
		    $aTermIDs = array_map(function($oTerm){
                return $oTerm->term_id;
            }, $aResults);

			$query = new WP_Query(array(
				'post_type' => 'listing',
				'post_status' => 'publish',
				'posts_per_page' => $rest,
				'tax_query' => array(
					array(
						'taxonomy' => 'listing_tag',
						'field'    => 'term_id',
						'terms'    => $aTermIDs
                    )
				)
			));

			if ( $query->have_posts() ){
			    while ($query->have_posts()){
				    $query->the_post();
				    $featuredImg = get_the_post_thumbnail_url($query->post->ID, 'thumbnail');
				    $aPosts[] = array(
					    'link' => get_permalink($query->post->ID),
					    'title'=> get_the_title($query->post->ID),
					    'img'  => empty($featuredImg) ? $defaultFeaturedImg : $featuredImg
				    );
                }
            }
            wp_reset_postdata();
        }

        if ( !empty($aPosts) ){
	        wp_send_json_success(array(
		        'msg' => $aPosts
	        ));
        }else{
	        wp_send_json_error(array(
		        'msg' => esc_html__('We found no results', 'wiloke-listgo-functionality')
	        ));
        }

    }

	public function addFieldsToSearchForm(){
		if ( !is_page_template('templates/half-map.php') && !is_page_template('templates/listing.php') && !is_tax('listing_cat') && !is_tax('listing_location') && !is_tax('listing_tag') ){
			return false;
		}

		global $wiloke;
		if ( !isset($wiloke->aThemeOptions['listing_toggle_search_by_tag']) || ($wiloke->aThemeOptions['listing_toggle_search_by_tag'] == 'enable') ) :
			$aTags = get_terms(
				array(
					'taxonomy'   => 'listing_tag',
					'hide_empty' => true,
					'orderby'    => apply_filters('listgo/search-field/tag/orderby', 'count'),
					'order'    => apply_filters('listgo/search-field/tag/order', 'DESC')
				)
			);

			if ( is_tax('listing_tag') ){
				$oTag = get_queried_object();
				$aSearching = array($oTag->term_id);
			}else{
				$aSearching = isset($_REQUEST['s_listing_tag']) ? $_REQUEST['s_listing_tag'] : '';
			}

			if ( isset($wiloke->aThemeOptions['listing_toggle_show_tags']) && $wiloke->aThemeOptions['listing_toggle_show_tags'] == 'disable' ){
			    $tagStatus = 'none';
            }else{
			    $tagStatus = 'block';
            }

			if ( !empty($aTags) && !is_wp_error($aTags) ) : ?>
                <div class="form-item item--tags">
                    <label class="label"><?php esc_html_e('Filter by tags', 'listgo'); ?> <span class="toggle-tags"></span></label>
                    <div class="item--tags-toggle" style="display: <?php echo esc_attr($tagStatus); ?>;">
						<?php foreach ( $aTags as $oTag ) :
							$checked = !empty($aSearching) && in_array($oTag->term_id, $aSearching) ? 'checked' : ''; ?>
                            <label for="s_listing_tag<?php echo esc_attr($oTag->term_id); ?>" class="input-checkbox">
                                <input id="s_listing_tag<?php echo esc_attr($oTag->term_id); ?>" name="s_listing_tag[]" class="listgo-filter-by-tag" type="checkbox" value="<?php echo esc_attr($oTag->term_id); ?>" <?php echo esc_attr($checked); ?>>
                                <span></span>
								<?php echo esc_html($oTag->name); ?>
                            </label>
						<?php endforeach; ?>
                    </div>
                </div>
				<?php
			endif;
		endif;
	}

	public static function searchForm($mapPage=null, $isShowAdvancedFilter=false, $atts=array(), $currentlySearchOn=null){
		global $wiloke, $post;
		if ( empty($post) ){
			return '';
		}
		
		$isShowAdvancedFilter = apply_filters('wiloke/listgo/admin/public/show_advanced_filter', $isShowAdvancedFilter);

		$inputName = 's_search';
		$pageID = '';
		if ( empty($mapPage) ){
			if ( !empty($wiloke->aThemeOptions) ){
				if ( $wiloke->aThemeOptions['listing_search_page'] === 'self' ){
					$mapPage = get_permalink($post->ID);
					$pageID = $post->ID;
				}else{
					if ( isset($wiloke->aThemeOptions['header_search_map_page']) && !empty($wiloke->aThemeOptions['header_search_map_page']) ){
						$mapPage = function_exists('wpml_object_id') ? get_permalink(wpml_object_id($wiloke->aThemeOptions['header_search_map_page']) ) : get_permalink($wiloke->aThemeOptions['header_search_map_page']);
						$pageID = function_exists('wpml_object_id') ? wpml_object_id($wiloke->aThemeOptions['header_search_map_page']) : $wiloke->aThemeOptions['header_search_map_page'];
					}else{
						$mapPage =  home_url('/');
						$inputName = 's';
					}
				}
			}else{
				$mapPage =  home_url('/');
				$inputName = 's';
			}
		}

		$aLocations = array();
		if ( $wiloke->aThemeOptions['listing_search_number_of_locations'] != 0 ){
			$aLocations = get_terms(
				array(
					'taxonomy'    => 'listing_location',
					'orderby'     => isset($wiloke->aThemeOptions['listing_search_location_order_by']) ? $wiloke->aThemeOptions['listing_search_location_order_by'] : 'id',
					'hide_empty'  => 1,
					'number'      => isset($wiloke->aThemeOptions['listing_search_number_of_locations']) ? $wiloke->aThemeOptions['listing_search_number_of_locations'] : 10,
					'order' => isset($wiloke->aThemeOptions['listing_search_location_order']) ? $wiloke->aThemeOptions['listing_search_location_order'] : 'DESC'
				)
			);
        }
		$aCategories = Wiloke::getTaxonomyHierarchy('listing_cat', 0, true, false, apply_filters('listgo/filter/searchForm/categories', array(
            'orderby' => isset($wiloke->aThemeOptions['listing_search_category_order_by']) ? $wiloke->aThemeOptions['listing_search_category_order_by'] : 'count',
            'number'      => isset($wiloke->aThemeOptions['listing_search_number_of_categories']) ? abs($wiloke->aThemeOptions['listing_search_number_of_categories']) : 10,
            'order' => isset($wiloke->aThemeOptions['listing_search_category_order']) ? $wiloke->aThemeOptions['listing_search_category_order'] : 'DESC'
        )));

		$searchFieldTitle = isset($atts['search_field_title']) && !empty($atts['search_field_title']) ? $atts['search_field_title'] : esc_attr($wiloke->aThemeOptions['header_search_keyword_label']);
		$locationFieldTitle = isset($atts['location_field_title']) && !empty($atts['location_field_title']) ? $atts['location_field_title'] : esc_attr($wiloke->aThemeOptions['header_search_location_label']);

		$s = isset($_REQUEST['s_search']) ? $_REQUEST['s_search'] : get_search_query();
		$s = isset($_REQUEST['cache_previous_search']) && !empty($_REQUEST['cache_previous_search']) ? $_REQUEST['cache_previous_search'] : $s;

		$catID = '';
		if ( isset($_REQUEST['s_listing_cat']) ){
			$catID = $_REQUEST['s_listing_cat'][0];
		}elseif ( is_tax('listing_cat') ){
			$catID = get_queried_object()->term_id;
			$s = single_term_title('', false);
		}

		$sWithinRadius = $wiloke->aThemeOptions['listgo_search_default_radius'];
		if ( isset($_REQUEST['s_within_radius']) && !empty($_REQUEST['s_within_radius']) ){
			$sWithinRadius =  trim($_REQUEST['s_within_radius']);
		}

		$sUnit = isset($wiloke->aThemeOptions['listgo_search_default_unit']) ? $wiloke->aThemeOptions['listgo_search_default_unit'] : 'km';
		if ( isset($_REQUEST['s_unit']) && !empty($_REQUEST['s_unit']) ){
			$sUnit = trim($_REQUEST['s_unit']);
		}
		$maxRadius = !isset($wiloke->aThemeOptions['listgo_search_max_radius']) ? 20 : $wiloke->aThemeOptions['listgo_search_max_radius'];
		$minRadius = !isset($wiloke->aThemeOptions['listgo_search_min_radius']) ? 1 : $wiloke->aThemeOptions['listgo_search_min_radius'];

		?>
		<form action="<?php echo esc_url($mapPage); ?>" method="GET" id="listgo-searchform" class="form form--listing <?php echo esc_attr($currentlySearchOn); ?>">
			<?php if ( isset($atts['alignment']) && ( $atts['alignment'] === 'not_center' || $atts['alignment'] === 'not_center_2' || $atts['alignment'] === 'not_center_3') && !empty($atts['search_form_title']) ) : ?>
				<h3 class="wiloke-searchform-title"><?php echo esc_html($atts['search_form_title']); ?></h3>
			<?php endif; ?>
			<input type="hidden" name="page_id" value="<?php echo esc_attr($pageID); ?>">
			<div class="form-item item--search">
				<label for="<?php echo esc_attr($inputName); ?>" class="label"><?php echo esc_html($searchFieldTitle); ?></label>
				<span class="input-text input-icon-inside">
                    <input id="<?php echo esc_attr($inputName); ?>" type="text" name="<?php echo esc_attr($inputName); ?>" value="<?php echo esc_attr(stripslashes($s)); ?>">
					<?php
					if ( !empty($aCategories) && !is_wp_error($aCategories) ) :
						?>
						<input type="hidden" id="wiloke-original-search-suggestion" value="<?php echo esc_attr(json_encode($aCategories)); ?>">
						<input type="hidden" id="s_listing_cat" name="s_listing_cat[]" value="<?php echo esc_attr($catID); ?>">
					<?php endif; ?>
					<i class="input-icon icon_search"></i>
                    <input type="hidden" id="cache_previous_search" name="cache_previous_search" value="">
                </span>
			</div>
			<?php
			if ( !empty($aLocations) && !is_wp_error($aLocations) ){
				$aLocations = json_encode($aLocations);
			}else{
				$aLocations = '';
			}
			$toggleFilterByPrice = !isset($wiloke->aThemeOptions['listing_toggle_filter_price']) || ($wiloke->aThemeOptions['listing_toggle_filter_price']=='enable');
			$toggleBusinessHour = !isset($wiloke->aThemeOptions['listing_toggle_business_hours']) || $wiloke->aThemeOptions['listing_toggle_business_hours'] == 'enable';

			$aRowsWitdh = array();
			$rowKey = 0;
			if ( !$toggleBusinessHour && !$toggleFilterByPrice ){
				$aRowsWitdh[] = '100%';
			}else if ( !$toggleBusinessHour || !$toggleFilterByPrice ){
				$aRowsWitdh[] = '66.66%';
				$aRowsWitdh[] = '33.33%';
			}else{
				$aRowsWitdh[] = '33.33%';
			}
			
			self::renderLocationField($locationFieldTitle, $aLocations);
			?>
			<?php if ( $isShowAdvancedFilter ) : ?>
				<?php if ( $toggleBusinessHour  ) : ?>
				<div class="form-item item--toggle-opennow" <?php if(!isset($aRowsWitdh[$rowKey])) : ?>style="width: <?php echo esc_attr($aRowsWitdh[$rowKey]); ?>;" <?php $rowKey = $rowKey+1; endif; ?>>
					<label for="s_opennow" class="checkbox-btn">
						<input id="s_opennow" type="checkbox" name="s_opennow" value="1">
						<span class="checkbox-btn-span"><i class="fa fa-clock-o"></i><?php esc_html_e('Open Now', 'listgo'); ?></span>
					</label>
				</div>
				<?php endif; ?>
				<div class="form-item item--toggle-highestrated" <?php if(!isset($aRowsWitdh[$rowKey])) : ?>style="width: <?php echo esc_attr($aRowsWitdh[$rowKey]); ?>;" <?php $rowKey++; endif; ?>>
					<label for="s_highestrated" class="checkbox-btn">
						<input id="s_highestrated" type="checkbox" name="s_highestrated" value="1">
						<span class="checkbox-btn-span"><i class="fa fa-star-o"></i><?php esc_html_e('Highest Rated', 'listgo'); ?></span>
					</label>
				</div>
				<?php if ( $toggleFilterByPrice ) : ?>
					<div class="form-item item--price">
						<?php
						$aPriceSegments = array_merge(
							array('all' => esc_html__('Cost - It doesn\'t matter', 'listgo')),
							$wiloke->aConfigs['frontend']['price_segmentation']
						);
						?>
						<span class="input-select2 input-icon-inside">
		                    <select id="s_price_segment" name="s_price_segment" class="js_select2" data-placeholder="<?php echo esc_html__('Cost', 'listgo'); ?>">
		                        <?php
		                        foreach ( $aPriceSegments as $price => $name ) :
			                        $name = isset($wiloke->aThemeOptions['header_search_'.$price.'_cost_label']) ? $wiloke->aThemeOptions['header_search_'.$price.'_cost_label'] : $name;
			                        ?>
			                        <option value="<?php echo esc_attr($price); ?>"><?php echo esc_html($name); ?></option>
		                        <?php endforeach; ?>
		                    </select>
		                </span>
					</div>
				<?php endif; ?>

				<div class="form-item item--radius">
					<div class="listgo-unit-wrapper">
						<label class="label" for="s_unit"><?php esc_html_e('Radius', 'listgo'); ?></label>
						<div class="listgo-unit">
							<i class="arrow_carrot-down"></i>
							<select id="s_unit" name="s_unit">
								<option value="km" <?php selected($sUnit, 'km'); ?>><?php esc_html_e('Kilometer', 'listgo'); ?></option>
								<option value="mi" <?php selected($sUnit, 'mi'); ?>><?php esc_html_e('Mile', 'listgo'); ?></option>
							</select>
						</div>
						<?php do_action('wiloke/listgo/admin/public/search-form/radius'); ?>
					</div>
					<div class="input-slider" data-max-radius="<?php echo esc_attr($maxRadius); ?>" data-min-radius="<?php echo esc_attr($minRadius); ?>" data-current-radius="<?php echo esc_attr($sWithinRadius); ?>">
						<input id="s_radius" name="s_radius" type="hidden" value="<?php echo esc_attr($sWithinRadius); ?>">
					</div>
				</div>
			<?php endif; ?>

			<?php do_action('wiloke/listgo/admin/public/search-form'); ?>

			<?php if ( isset($atts['is_hero_shortcode']) ) : ?>
			<input id="wiloke-is-hero-sc" type="hidden" name="wiloke_is_hero_sc" value="1">
			<?php endif; ?>

			<div class="form-item item--submit">
				<input type="submit" value="<?php esc_html_e('Search Now', 'listgo'); ?>" />
			</div>
		</form>
		<?php
	}

	public static function renderLocationField($locationFieldTitle, $aLocations){
		global $wiloke;
		$termID = '';
		$termName = '';
		if ( isset($_REQUEST['location_term_id']) ){
			$termID = $_REQUEST['location_term_id'];
			$termName = $_GET['s_listing_location'];
		}elseif ( is_tax('listing_location') ){
			$termID = get_queried_object()->term_id;
			$termName = single_term_title('', false);
		}

		$autoLocationClass = isset($wiloke->aThemeOptions['listing_search_toggle_autolocation']) && $wiloke->aThemeOptions['listing_search_toggle_autolocation'] == 'disable' ? 'not-auto-location-by-google' : 'auto-location-by-google';
		?>
		<div class="form-item item--localtion">
			<label for="s_listing_location" class="label"><?php echo esc_html($locationFieldTitle); ?></label>
			<span class="input-text input-icon-inside">
                <input type="text" id="s_listing_location" class="<?php echo esc_attr($autoLocationClass); ?>" name="s_listing_location" value="<?php echo esc_attr($termName); ?>">
                <input type="hidden" id="s-listing-location-suggestion" value="<?php echo esc_attr($aLocations); ?>">
                <input type="hidden" id="s-location-place-id" name="location_place_id" value="<?php echo isset($_REQUEST['location_place_id']) ? esc_attr($_REQUEST['location_place_id']) : ''; ?>">
                <input type="hidden" id="s-location-latitude-longitude-id" name="location_latitude_longitude" value="<?php echo isset($_REQUEST['location_latitude_longitude']) ? esc_attr($_REQUEST['location_latitude_longitude']) : ''; ?>">
                <input type="hidden" id="s-location-term-id" name="location_term_id" value="<?php echo esc_attr($termID); ?>">
                <i id="listgo-current-location" class="input-icon icon_pin_alt"></i>
            </span>
		</div>
		<?php
	}

	public function searchSuggestion(){
		$aData = $_GET;

		if ( !isset($aData['s']) || empty($aData['s']) ){
			wp_send_json_error(array('message'=>esc_html__('Please enter in your destination', 'listgo')));
		}

		$aArgs = array(
			's'                 => stripslashes($aData['s']),
			'posts_per_page'    => !empty($aData['is_hero_sc']) ? -1 : 5,
			'post_type'         => 'listing',
			'post_status'       => 'publish'
		);

		$aTaxQuery = array();
		$nothingFoundText = apply_filters('wiloke/listgo/nothingFoundText', esc_html__('Nothing found. Please try another keyword.', 'listgo'));

		if ( isset($aData['latLng']) && !empty($aData['latLng']) ){
			$aLatLng = explode(',', $aData['latLng']);
			$distance = isset($aData['sWithin']) ? abs($aData['sWithin']) :  10;
			$unit = isset($aData['sUnit']) ? trim($aData['sUnit']) :  'km';
			$aListingInRadius = \WilokeListgoFunctionality\Model\GeoPosition::searchLocationWithin(trim($aLatLng[0]), trim($aLatLng[1]), $distance, $unit);
			if ( empty($aListingInRadius) ){
				wp_send_json_error(array('message'=>$nothingFoundText));
			}

			$aArgs['post__in'] = $aListingInRadius['IDs'];
		}else{
			if ( isset($aData['listing_locations']) && !empty($aData['listing_locations']) ){
				$aData['listing_locations'] = isset($aData['location_term_id']) && !empty($aData['location_term_id']) ? $aData['location_term_id'] : $aData['listing_locations'];

				$aLocationArgs = WilokePublic::parseLocationQuery($aData);
				if ( empty($aLocationArgs) ){
					wp_send_json_error(array('message'=>$nothingFoundText));
				}else{
					$aTaxQuery[] = $aLocationArgs;
				}
			}
		}

		if ( isset($aData['listing_cats']) && !empty($aData['listing_cats']) && ($aData['listing_cats'] !== 'all') ){
			$aTaxQuery[] = array(
				'taxonomy' => 'listing_cat',
				'field'    => 'term_id',
				'terms'    => $aData['listing_cats']
			);
		}

		if ( isset($aData['listing_tags']) && !empty($aData['listing_tags']) && ($aData['listing_tags'] !== 'all') ){
			$aTaxQuery[] = array(
				'taxonomy' => 'listing_tag',
				'field'    => 'term_id',
				'terms'    => $aData['listing_tags']
			);
		}

		if ( !empty($aTaxQuery) ){
			if ( count($aTaxQuery) > 1 ){
				$aArgs['tax_query']['relation'] = 'AND';
			}
			$aArgs['tax_query'][] = $aTaxQuery;
		}

		$aArgs['order'] = 'DESC';
		$aArgs['orderby'] = 'menu_order post_date';

		$query = new WP_Query($aArgs);

		$aInfo = array();

		if ( $query->have_posts() ){
			while($query->have_posts()){
				$query->the_post();
				$aInfo[$query->post->ID] = WilokeListingLayout::listingInfo($query->post);
			}
			wp_send_json_success($aInfo);
		}else{
			wp_send_json_error(array('message'=>$nothingFoundText));
		}
	}
}