/**
 * Live Preview get Lat & Long
 *
 * @since 1.0
 * @author Wiloke
 */
(function ($) {
    "use strict";

	function decimalAdjust(type, value, exp) {
		// If the exp is undefined or zero...
		if (typeof exp === 'undefined' || +exp === 0) {
			return Math[type](value);
		}
		value = +value;
		exp = +exp;
		// If the value is not a number or the exp is not an integer...
		if (isNaN(value) || !(typeof exp === 'number' && exp % 1 === 0)) {
			return NaN;
		}
		// Shift
		value = value.toString().split('e');
		value = Math[type](+(value[0] + 'e' + (value[1] ? (+value[1] - exp) : -exp)));
		// Shift back
		value = value.toString().split('e');
		return +(value[0] + 'e' + (value[1] ? (+value[1] + exp) : exp));
	}

	// Decimal round
	if (!Math.round10) {
		Math.round10 = function(value, exp) {
			return decimalAdjust('round', value, exp);
		};
	}

    $.fn.WilokeMapExtendAdmin = function (opts) {
        let self = this,
            defaults = {
                initMap: function () {},
                beforeResize: function () {},
                afterResize: function () {},
                dragMarker : function () {},
                dragEndMarker: function () {}
            },
            options = $.extend(defaults, opts);
            self.WilokeMapAdmin = {
                options : options,
	            focusDetectMyLocation: false,
	            oPlaces: null,
	            oLatLng: {
		            lat: 48.856614,
		            lng: 2.352222
	            },
                el: self,
                init: function () {
                    this.stopParseGeocode = false;
                    this.oTranslation = typeof WILOKE_GLOBAL !== 'undefined' ? WILOKE_LISTGO_TRANSLATION : {
	                    noresults: 'No results found',
	                    geocodefailed: 'Geocoder failed due to:',
                    };
                    this.createHTML();
                    this.getCurrentPosition();
                    this.createMap();
                },
                getCurrentPosition: function(){
                    if ( $('#wiloke-latlong').length && $('#wiloke-latlong').val() == '' ){
                        let oCurrentLatLng = $('body').data('wiloke_listgo_user_latlng'), self = this;

                        if ( typeof oCurrentLatLng !== 'undefined' ){
                            this.oLatLng.lat =  oCurrentLatLng.lat;
                            this.oLatLng.lng =  oCurrentLatLng.lng;
                            this.setCurrentLocation();
                        }else{
                            this.getGeoLocation();
                        }
                    }
                    
                    $('#wiloke-fill-my-location').on('click', ()=>this.getGeoLocation());
                },
                setCurrentLocation(){
                    let self = this;
                    let myLatLng = new google.maps.LatLng(self.oLatLng.lat, self.oLatLng.lng);
                    self.marker.setPosition(myLatLng);
                    self.infoWindow.setContent(self.oLatLng.lat + ',' + self.oLatLng.lng);
                    self.infoWindow.open(self.map, self.marker);
                    
                    self.map.setCenter(myLatLng);
                    self.map.panTo(myLatLng); 

                    self.geocoder.geocode({
                        latLng: myLatLng
                    }, function(responses) {
                        if (responses && responses.length > 0) {
                            self.marker.formatted_address = responses[0].formatted_address;
                        } else {
                            self.marker.formatted_address = 'Cannot determine address at this location.';
                        }
                        
                        self.inputLocation.val(self.marker.formatted_address);
                    });
                },
                getGeoLocation(){
                    let self = this;
                    if (navigator.geolocation) {
                        navigator.geolocation.getCurrentPosition(oPosition=>{
                            self.oLatLng.lat = oPosition.coords.latitude;
                            self.oLatLng.lng = oPosition.coords.longitude;
                            self.inputLatLong.attr('value', self.oLatLng.lat +','+self.oLatLng.lng);
                            
                            if ( typeof self.map !== 'undefined' ){
                                self.setCurrentLocation();
                            }
                        });
                    }
                },
                createHTML: function () {
                    this.wrapperMap = $('<div class="pi-map-wrapper"></div>');
                    this.iconMarker = $('<div class="icon-marker"></div>');
                    this.inputLocation = $('#wiloke-location');
                    this.inputLatLong  = $("#wiloke-latlong");
                    this.inputWebsite = $('#listing_website');
                    this.inputPlaceInfo = $('#wiloke-place-information');
                    this.inputPhone = $('#listing_phone');
                    this.$usingMyLocation = $('#wiloke-fill-my-location');
	                this.$listingPreview = $('.add-listing-group-preview'),
		            this.$mapPreview = $('.add-listing-group-preview-map'),
                    this.el.append(this.wrapperMap);
                    this.el.append(this.searchButton);
                    this.el.append(this.iconMarker);
                },
                mapStyle(){
                    return  new google.maps.StyledMapType([
                        {elementType: 'geometry', stylers: [{color: '#ebe3cd'}]},
                        {elementType: 'labels.text.fill', stylers: [{color: '#523735'}]},
                        {elementType: 'labels.text.stroke', stylers: [{color: '#f5f1e6'}]},
                        {
                            featureType: 'administrative',
                            elementType: 'geometry.stroke',
                            stylers: [{color: '#c9b2a6'}]
                        },
                        {
                            featureType: 'administrative.land_parcel',
                            elementType: 'geometry.stroke',
                            stylers: [{color: '#dcd2be'}]
                        },
                        {
                            featureType: 'administrative.land_parcel',
                            elementType: 'labels.text.fill',
                            stylers: [{color: '#ae9e90'}]
                        },
                        {
                            featureType: 'landscape.natural',
                            elementType: 'geometry',
                            stylers: [{color: '#dfd2ae'}]
                        },
                        {
                            featureType: 'poi',
                            elementType: 'geometry',
                            stylers: [{color: '#dfd2ae'}]
                        },
                        {
                            featureType: 'poi',
                            elementType: 'labels.text.fill',
                            stylers: [{color: '#93817c'}]
                        },
                        {
                            featureType: 'poi.park',
                            elementType: 'geometry.fill',
                            stylers: [{color: '#a5b076'}]
                        },
                        {
                            featureType: 'poi.park',
                            elementType: 'labels.text.fill',
                            stylers: [{color: '#447530'}]
                        },
                        {
                            featureType: 'road',
                            elementType: 'geometry',
                            stylers: [{color: '#f5f1e6'}]
                        },
                        {
                            featureType: 'road.arterial',
                            elementType: 'geometry',
                            stylers: [{color: '#fdfcf8'}]
                        },
                        {
                            featureType: 'road.highway',
                            elementType: 'geometry',
                            stylers: [{color: '#f8c967'}]
                        },
                        {
                            featureType: 'road.highway',
                            elementType: 'geometry.stroke',
                            stylers: [{color: '#e9bc62'}]
                        },
                        {
                            featureType: 'road.highway.controlled_access',
                            elementType: 'geometry',
                            stylers: [{color: '#e98d58'}]
                        },
                        {
                            featureType: 'road.highway.controlled_access',
                            elementType: 'geometry.stroke',
                            stylers: [{color: '#db8555'}]
                        },
                        {
                            featureType: 'road.local',
                            elementType: 'labels.text.fill',
                            stylers: [{color: '#806b63'}]
                        },
                        {
                            featureType: 'transit.line',
                            elementType: 'geometry',
                            stylers: [{color: '#dfd2ae'}]
                        },
                        {
                            featureType: 'transit.line',
                            elementType: 'labels.text.fill',
                            stylers: [{color: '#8f7d77'}]
                        },
                        {
                            featureType: 'transit.line',
                            elementType: 'labels.text.stroke',
                            stylers: [{color: '#ebe3cd'}]
                        },
                        {
                            featureType: 'transit.station',
                            elementType: 'geometry',
                            stylers: [{color: '#dfd2ae'}]
                        },
                        {
                            featureType: 'water',
                            elementType: 'geometry.fill',
                            stylers: [{color: '#b9d3c2'}]
                        },
                        {
                            featureType: 'water',
                            elementType: 'labels.text.fill',
                            stylers: [{color: '#92998d'}]
                        }
                    ], {name: 'Styled Map'});
                },
                createMap: function () {
                    // Create default bounds
                    let defaultBounds, latDef = '', longDef = '', latLngDef, parseLatLong;
                    this.geocoder = new google.maps.Geocoder();
                    this.location = new google.maps.LatLng(this.oLatLng);
                    this.mapOptions =  {
                        center: this.location,
	                    zoom: 7,
	                    mapTypeControlOptions: {
		                    mapTypeIds: ['roadmap', 'satellite', 'hybrid', 'terrain',
			                    'styled_map']
	                    }
                    };

                    this.map = new google.maps.Map(this.wrapperMap[0], this.mapOptions);
	                this.map.mapTypes.set('styled_map', this.mapStyle());
	                this.map.setMapTypeId('styled_map');
                    this.map.setZoom(16);

                    // Create Marker
                    this.marker = new google.maps.Marker ({
                        map: this.map,
                        draggable: true
                    });

                    // Create info Window
                    this.infoWindow =  new google.maps.InfoWindow();

                    if ( !_.isUndefined(this.inputLatLong.val()) && !_.isEmpty(this.inputLatLong.val()) ){
                        parseLatLong = this.inputLatLong.val().split(',');
                        latDef  = parseLatLong[0];
                        longDef = parseLatLong[1];
                    }

                    if ( latDef !== '' && longDef !== '' )
                    {
                        latLngDef = new google.maps.LatLng(latDef, longDef);
                        this.marker.setPosition(latLngDef);
                        this.infoWindow.setContent(latDef + ',' + longDef);
                        this.infoWindow.open(this.map, this.marker);
                    }

                    // this.map.fitBounds(defaultBounds);

                    // Create the search box and link it to the UI element.

                    /*pirateS*/
                   
                    this.search = new google.maps.places.Autocomplete(document.getElementById('wiloke-location'));

	                this.listenEvent();

					// Callback beforeResize
                    // options.initMap(this.el, options);
                    // End Callback
                },
                setPhone: function (place) {
					if ( this.inputPhone.length && this.inputPhone.val() === '' ){
						this.inputPhone.val(place.international_phone_number);
					}
                },
	            setWebsite: function (place) {
		            if ( this.inputWebsite.length && this.inputWebsite.val() === '' ){
			            this.inputWebsite.val(place.website);
		            }
	            },
	            b64EncodeUnicode: function(str) {
				    return btoa(encodeURIComponent(str).replace(/%([0-9A-F]{2})/g, function(match, p1) {
					    return String.fromCharCode('0x' + p1);
				    }));
			    },
                setPlaceInfo: function (place) {
	                if ( this.inputPlaceInfo.length ){
		                this.inputPlaceInfo.val(this.b64EncodeUnicode(JSON.stringify(place)));
	                }
                },
                geocodePosition: function(pos) {
                    let _this = this;

                    if ( _this.stopParseGeocode ){
                        _this.stopParseGeocode = false;
                        return;
                    }

                    _this.geocoder.geocode({
                        latLng: pos
                    }, function(responses) {
                        if (responses && responses.length > 0) {
                            _this.marker.formatted_address = responses[0].formatted_address;
                        } else {
                            _this.marker.formatted_address = 'Cannot determine address at this location.';
                        }

                        _this.inputLocation.val(_this.marker.formatted_address);
                        _this.inputLocation.trigger('updateLatLng', {changeLocation: true});
                    });
                },
                listenEvent : function () {
                    let _this = this;

                    _this.inputLocation.on('updateLatLng', function (event, action) {
                        if (_this.location && action && action.changeLocation === true) {
                            let lat = Math.round10(_this.location.lat(), -4),
                                long = Math.round10(_this.location.lng(), -4);
                            _this.inputLatLong.attr('value', lat+','+long);

	                        _this.resize();
                            if ( !_.isEmpty(_this.oPlaces) ){
	                            _this.setPhone(_this.oPlaces);
	                            _this.setWebsite(_this.oPlaces);
	                            _this.setPlaceInfo(_this.oPlaces);
                            }
                        }
                    });

                    // Listen for the event fired when the user selects an item from the
                    // pick list. Retrieve the matching places for that item.
	                google.maps.event.addListener(_this.search, 'place_changed', function() {
                        // let places = _this.search.getPlaces();
		                let place = _this.search.getPlace();

                        // For each place, get the icon, place name, and location.
                        // let bounds = new google.maps.LatLngBounds();
		                _this.location = place.geometry.location;
		                _this.oPlaces = place;

                        // _this.map.fitBounds(bounds);
                        _this.marker.setPosition(_this.location);
                        _this.stopParseGeocode = true;
                        google.maps.event.trigger (_this.marker, 'dragend');
                        /*Add Location: pirateS*/
                    });

                    // Bias the SearchBox results towards places that are within the bounds of the
                    // current map's viewport.
                    // google.maps.event.addListener(_this.map, 'bounds_changed', function() {
                    //     let bounds = _this.map.getBounds();
                    //     _this.search.setBounds(bounds);
                    // });

                    google.maps.event.addListener(_this.marker, 'dragend', function() {
                        options.dragMarker();
                    });

                    google.maps.event.addListener(_this.marker, 'dragend', function() {
                        _this.location = _this.marker.getPosition();

                        _this.geocodePosition(_this.location);
                        let lat = Math.round10(parseFloat(_this.location.lat()), -4),
	                        long = Math.round10(parseFloat(_this.location.lng()), -4);

                        _this.infoWindow.setContent(lat + ', ' + long);
                        _this.infoWindow.open(_this.map, _this.marker);
                        options.dragEndMarker();
	                    _this.inputLocation.trigger('updateLatLng', {changeLocation: true});
                    });

                    google.maps.event.addDomListener(window, 'resize', function() {
                        _this.resize();
                    });
                },
                resize : function () {
                    // Callback beforeResize
                    options.beforeResize(this.el, options);
                    // End Callback

                    google.maps.event.trigger(this.map, "resize");
                    this.map.setCenter(this.location);

                    // Callback afterResize
                    options.afterResize(this.el, options);
                    // End Callback afterResize
                }
            };
            self.WilokeMapAdmin.init();
    };

    $(window).load(function(){
        let $wilokeMap = $('#wiloke-map');

        if ( $wilokeMap.length ){
	        $wilokeMap.WilokeMapExtendAdmin({
		        initMap: function () {},
		        beforeResize: function () {},
		        afterResize: function () {},
		        dragMarker : function () {},
		        dragEndMarker: function () {}
	        });
        }

    })
})(jQuery);